@@ -17,7 +17,6 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -34,7 +33,7 @@ void debug(const char *format, const char *file, const int line,
    va_list ap;
    char *buffer, *buffer_next;
    int n, size = 100;
-   
+
    if (halevt_fork)
    {
       va_start (ap, func);
@@ -59,19 +58,23 @@ void debug(const char *format, const char *file, const int line,
           size = n+1; /* precisely what is needed */
       else           /* glibc 2.0 */
           size *= 2;  /* twice the old size */
-      if ((buffer_next = realloc (buffer, size)) == NULL) 
+      if ((buffer_next = realloc (buffer, size)) == NULL)
       {
          free(buffer);
          return;
-      } 
-      else 
+      }
+      else
       {
           buffer = buffer_next;
       }
    }
    size = strlen(buffer) + strlen(file) + strlen(func) + 26;
    buffer_next = malloc(sizeof(char *)* size);
-   if (buffer_next == NULL) { return; }
+   if (buffer_next == NULL)
+   {
+      free(buffer);
+      return;
+   }
    if (halevt_report)
    {
       snprintf(buffer_next, size - 1, "%s", buffer);
@@ -79,9 +82,11 @@ void debug(const char *format, const char *file, const int line,
    }
    else
    {
-      snprintf(buffer_next, size - 1, "%s:%d (%s) %s", 
+      snprintf(buffer_next, size - 1, "%s:%d (%s) %s",
          file, line, func, buffer);
       buffer_next[size-1] = '\0';
       fprintf (stderr, "%s\n", buffer_next);
    }
+   free(buffer);
+   free(buffer_next);
 }
